#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>


typedef struct _node {
	int value;
	struct _node *left;
	struct _node *right;
} node;


node* createNode(int value) {
	node *new = (node*) malloc(sizeof(node));
	new->value = value;
	new->left = NULL;
	new->right = NULL;

	return new;
}

void clear(node **root) {
	if (*root) {
		clear(&((*root)->left));
		clear(&((*root)->right));
		free(*root);

		*root = NULL;
	}
}

int isLeaf(node *theNode) {
	return theNode->left == NULL && theNode->right == NULL;
}

node* getMin(node *root) {
	if (root) {
		if (!root->left) {
			return root;
		} else {
			return getMin(root->left);
		}
	}

	return NULL;
}

node* getMax(node *root) {
	if (root) {
		if (!root->right) {
			return root;
		} else {
			return getMax(root->right);
		}
	}

	return NULL;
}

void addNode(node **root, int value) {
	if (*root == NULL) {
		*root = createNode(value, NULL, NULL);
	} else if ((*root)->value > value) {
		addNode(&((*root)->left), value);
	} else {
		addNode(&((*root)->right), value);
	}
}

void deleteNode(node **root, int value) {
	if (*root == NULL) {
		return;
	}

	if ((*root)->value == value) {
		if (isLeaf(*root)) {
			clear(root);
		} else {
			node *temp;
			if ((*root)->left == NULL) {
				temp = *root;
				*root = (*root)->right;
			} else if ((*root)->right == NULL) {
				temp = *root;
				*root = (*root)->left;
			} else {
				/* Nadje se najmanji broj u desnom podstablu i samo se digne na mesto odakle se brise ovaj cvor,
				 * pa zatim obrise. */
				temp = getMax((*root)->left);
				(*root)->value = temp->value;

				// Mora rekurzivno da se obrise vrednost, zbog toga sto se ne raskida veza od cvora koji prethodi minimalnom, ka minimalnom.
				deleteNode(&(*root)->right, temp->value);
			}
		}
	} else if ((*root)->value > value) {
		deleteNode(&((*root)->left), value);
	} else {
		deleteNode(&((*root)->right), value);
	}
}

int sum(node *root) {
	if (!root) {
		return 0;
	} else {
		return root->value + sum(root->left) + sum(root->right);
	}
}

int sumOfLeafs(node *root) {
	if (!root) {
		return 0;
	}

	if (isLeaf(root)) {
		return root->value;
	} else {
		return sumOfLeafs(root->left) + sumOfLeafs(root->right);
	}
}

node* find(node *root, int value) {
	if (root) {
		if (root->value == value) {
			return root;
		}

		if (root->value < value) {
			return find(root->right, value);
		}

		return find(root->left, value);
	}

	return NULL;
}

int contains(node *root, int value) {
	return find(root, value) != NULL;
}

int depth(node *root) {
	if (!root) {
		return 0;
	}

	int depthOfLeft = depth(root->left);
	int depthOfRight = depth(root->right);
	return 1 + MAX(depthOfLeft, depthOfRight);
}

void infix(node *root) {
	if (root) {
		infix(r#include <stdio.h>

#include "arr_queue.h"

oot->left);
		printf("%d ", root->value);
		infix(root->right);
	}
}
/*           7
 *       6      10
 *                  15
 *    3                 */
/* popraviti da lepse stampa stablo */
void prettyPrintHelper(node *root, node *fromNode) {
	int depthOfRoot = depth(root);
	if (fromNode) {
		int i = 0;
		for (; i <= depthOfRoot - depth(fromNode); i++) {
			printf("-");
		}
		printf(" %d\n", fromNode->value);

		prettyPrintHelper(root, fromNode->left);
		prettyPrintHelper(root, fromNode->right);
	}
}

void prettyPrint(node *root) {
	prettyPrintHelper(root, root);
}

node* construct_tree_helper(int preorder[], int size, int *preIndex, int left,
		int right) {
	if (*preIndex > size || left > right) {
		return NULL;
	}

	node *root = create_node(preorder[*preIndex]);
	*preIndex = *preIndex + 1;

	if (left == right) {
		return root;
	}

	//Pronalazi se granica, prvi element koji je veci od korena
	int i = left;
	for (; i <= right; i++) {
		if (preorder[i] > root->value) {
			break;
		}
	} // pronadjen je prvi veci element
	root->left = construct_tree_helper(preorder, size, preIndex, *preIndex,
			i - 1);
	root->right = construct_tree_helper(preorder, size, preIndex, i, right);

	return root;

}

node* construct_tree(int preorder[], int size) {
	int preIndex = 0;
	return construct_tree_helper(preorder,size, &preIndex, 0, size - 1 );
}

void print_inorder(node *tree) {
	if (tree == NULL) {
		return;
	}
	print_inorder(tree->left);
	printf("%d", tree->value);
	print_inorder(tree->right);
}



int main() {
	node *root = createNode(-5, NULL, NULL);
	addNode(&root, -10);
	addNode(&root, 9);
	addNode(&root, 7);
	addNode(&root, 11);
	addNode(&root, -8);
	addNode(&root, 10);
	addNode(&root, 12);

	/*               -5

	 -10               9
	 -8       7        11
	 10      12 */

	infix(root);

	printf("\nContains: %s\n", contains(root, 9) ? "yes" : "no");

	node *min = getMin(root);
	if (min != NULL) {
		printf("Min: %d\n", min->value);
	}

	node *max = getMax(root);
	if (max != NULL) {
		printf("MAx: %d\n", max->value);
	}

	printf("Sum: %d\n", sum(root));
	printf("Sum of leafs: %d\n", sumOfLeafs(root));

	printf("Depth: %d\n", depth(root));
	printf("Depth of -8: %d\n", depth(find(root, -8)));

	prettyPrint(root);

	 deleteNode(&root, 9);

	 prettyPrint(root);


	 int preorder_set[] = { 10, 5, 1, 7, 40, 50 };
	 int size = sizeof(preorder_set) / sizeof(preorder_set[0]);

	 node *tree = construct_tree(preorder_set, size);
	 printf("Inorder obilazak kroz konstruisano stablo: \n");
	 print_inorder(tree);
	 prettyPrint(tree);

	clear(&root);
	clear(&tree);

	return 0;
}
